
from epyk.core.py import PyDates

print(PyDates.PyDates().today)
print(PyDates.PyDates().now)
print(PyDates.PyDates().cob)
print(PyDates.PyDates().month_end)
print(PyDates.PyDates().months)
print(PyDates.PyDates().date_from_alias("T", "2019-08-08"))