"""
Related Pages:

		https://www.w3schools.com/charsets/ref_utf_currency.asp
"""

EURO_CURRENCY_SIGN = '&#8352'
COLON_SIGN = '&#8353'
CRUZEIRO_SIGN = '&#8354'
FRENCH_FRANC_SIGN = '&#8355'
LIRA_SIGN = '&#8356'
MILL_SIGN = '&#8357'
NAIRA_SIGN = '&#8358'
PESETA_SIGN = '&#8359'
RUPEE_SIGN = '&#8360'
WON_SIGN = '&#8361'
NEW_SHEQEL_SIGN = '&#8362'
DONG_SIGN = '&#8363'
EURO_SIGN = '&#8364'
KIP_SIGN = '&#8365'
TUGRIK_SIGN = '&#8366'
DRACHMA_SIGN = '&#8367'
GERMAN_PENNY_SYMBOL = '&#8368'
PESO_SIGN = '&#8369'
GUARANI_SIGN = '&#8370'
AUSTRAL_SIGN = '&#8371'
HRYVNIA_SIGN = '&#8372'
CEDI_SIGN = '&#8373'
LIVRE_TOURNOIS_SIGN = '&#8374'
SPESMILO_SIGN = '&#8375'
TENGE_SIGN = '&#8376'
INDIAN_RUPEE_SIGN = '&#8377'
TURKISH_LIRA_SIGN = '&#8378'
NORDIC_MARK_SIGN = '&#8379'
MANAT_SIGN = '&#8380'
RUBLE_SIGN = '&#8381'
LARI_SIGN = '&#8382'
BITCOIN_SIGN = '&#8383'
