"""
Related Pages:

		https://www.w3schools.com/charsets/ref_utf_math.asp
"""

FOR_ALL = '&#8704'
COMPLEMENT = '&#8705'
PARTIAL_DIFFERENTIAL = '&#8706'
THERE_EXISTS = '&#8707'
THERE_DOES_NOT_EXIST = '&#8708'
EMPTY_SET = '&#8709'
INCREMENT = '&#8710'
NABLA = '&#8711'
ELEMENT_OF = '&#8712'
NOT_AN_ELEMENT_OF = '&#8713'
SMALL_ELEMENT_OF = '&#8714'
CONTAINS_AS_MEMBER = '&#8715'
DOES_NOT_CONTAIN_AS_MEMBER = '&#8716'
SMALL_CONTAINS_AS_MEMBER = '&#8717'
END_OF_PROOF = '&#8718'
N_ARY_PRODUCT = '&#8719'
N_ARY_COPRODUCT = '&#8720'
N_ARY_SUMMATION = '&#8721'
MINUS_SIGN = '&#8722'
MINUS_OR_PLUS_SIGN = '&#8723'
DOT_PLUS = '&#8724'
DIVISION_SLASH = '&#8725'
SET_MINUS = '&#8726'
ASTERISK_OPERATOR = '&#8727'
RING_OPERATOR = '&#8728'
BULLET_OPERATOR = '&#8729'
SQUARE_ROOT = '&#8730'
CUBE_ROOT = '&#8731'
FOURTH_ROOT = '&#8732'
PROPORTIONAL_TO = '&#8733'
INFINITY = '&#8734'
RIGHT_ANGLE = '&#8735'
ANGLE = '&#8736'
MEASURED_ANGLE = '&#8737'
SPHERICAL_ANGLE = '&#8738'
DIVIDES = '&#8739'
DOES_NOT_DIVIDE = '&#8740'
PARALLEL_TO = '&#8741'
NOT_PARALLEL_TO = '&#8742'
LOGICAL_AND = '&#8743'
LOGICAL_OR = '&#8744'
INTERSECTION = '&#8745'
UNION = '&#8746'
INTEGRAL = '&#8747'
DOUBLE_INTEGRAL = '&#8748'
TRIPLE_INTEGRAL = '&#8749'
CONTOUR_INTEGRAL = '&#8750'
SURFACE_INTEGRAL = '&#8751'
VOLUME_INTEGRAL = '&#8752'
CLOCKWISE_INTEGRAL = '&#8753'
CLOCKWISE_CONTOUR_INTEGRAL = '&#8754'
ANTICLOCKWISE_CONTOUR_INTEGRAL = '&#8755'
THEREFORE = '&#8756'
BECAUSE = '&#8757'
RATIO = '&#8758'
PROPORTION = '&#8759'
DOT_MINUS = '&#8760'
EXCESS = '&#8761'
GEOMETRIC_PROPORTION = '&#8762'
HOMOTHETIC = '&#8763'
TILDE_OPERATOR = '&#8764'
REVERSED_TILDE = '&#8765'
INVERTED_LAZY_S = '&#8766'
SINE_WAVE = '&#8767'
WREATH_PRODUCT = '&#8768'
NOT_TILDE = '&#8769'
MINUS_TILDE = '&#8770'
ASYMPTOTICALLY_EQUAL_TO = '&#8771'
NOT_ASYMPTOTICALLY_EQUAL_TO = '&#8772'
APPROXIMATELY_EQUAL_TO = '&#8773'
APPROXIMATELY_BUT_NOT_ACTUALLY_EQUAL_TO = '&#8774'
NEITHER_APPROXIMATELY_NOR_ACTUALLY_EQUAL_TO = '&#8775'
ALMOST_EQUAL_TO = '&#8776'
NOT_ALMOST_EQUAL_TO = '&#8777'
ALMOST_EQUAL_OR_EQUAL_TO = '&#8778'
TRIPLE_TILDE = '&#8779'
ALL_EQUAL_TO = '&#8780'
EQUIVALENT_TO = '&#8781'
GEOMETRICALLY_EQUIVALENT_TO = '&#8782'
DIFFERENCE_BETWEEN = '&#8783'
APPROACHES_THE_LIMIT = '&#8784'
GEOMETRICALLY_EQUAL_TO = '&#8785'
APPROXIMATELY_EQUAL_TO_OR_THE_IMAGE_OF = '&#8786'
IMAGE_OF_OR_APPROXIMATELY_EQUAL_TO = '&#8787'
COLON_EQUALS = '&#8788'
EQUALS_COLON = '&#8789'
RING_IN_EQUAL_TO = '&#8790'
RING_EQUAL_TO = '&#8791'
CORRESPONDS_TO = '&#8792'
ESTIMATES = '&#8793'
EQUIANGULAR_TO = '&#8794'
STAR_EQUALS = '&#8795'
DELTA_EQUAL_TO = '&#8796'
EQUAL_TO_BY_DEFINITION = '&#8797'
MEASURED_BY = '&#8798'
QUESTIONED_EQUAL_TO = '&#8799'
NOT_EQUAL_TO = '&#8800'
IDENTICAL_TO = '&#8801'
NOT_IDENTICAL_TO = '&#8802'
STRICTLY_EQUIVALENT_TO = '&#8803'
LESS_THAN_OR_EQUAL_TO = '&#8804'
GREATER_THAN_OR_EQUAL_TO = '&#8805'
LESS_THAN_OVER_EQUAL_TO = '&#8806'
GREATER_THAN_OVER_EQUAL_TO = '&#8807'
LESS_THAN_BUT_NOT_EQUAL_TO = '&#8808'
GREATER_THAN_BUT_NOT_EQUAL_TO = '&#8809'
MUCH_LESS_THAN = '&#8810'
MUCH_GREATER_THAN = '&#8811'
BETWEEN = '&#8812'
NOT_EQUIVALENT_TO = '&#8813'
NOT_LESS_THAN = '&#8814'
NOT_GREATER_THAN = '&#8815'
NEITHER_LESS_THAN_NOR_EQUAL_TO = '&#8816'
NEITHER_GREATER_THAN_NOR_EQUAL_TO = '&#8817'
LESS_THAN_OR_EQUIVALENT_TO = '&#8818'
GREATER_THAN_OR_EQUIVALENT_TO = '&#8819'
NEITHER_LESS_THAN_NOR_EQUIVALENT_TO = '&#8820'
NEITHER_GREATER_THAN_NOR_EQUIVALENT_TO = '&#8821'
LESS_THAN_OR_GREATER_THAN = '&#8822'
GREATER_THAN_OR_LESS_THAN = '&#8823'
NEITHER_LESS_THAN_NOR_GREATER_THAN = '&#8824'
NEITHER_GREATER_THAN_NOR_LESS_THAN = '&#8825'
PRECEDES = '&#8826'
SUCCEEDS = '&#8827'
PRECEDES_OR_EQUAL_TO = '&#8828'
SUCCEEDS_OR_EQUAL_TO = '&#8829'
PRECEDES_OR_EQUIVALENT_TO = '&#8830'
SUCCEEDS_OR_EQUIVALENT_TO = '&#8831'
DOES_NOT_PRECEDE = '&#8832'
DOES_NOT_SUCCEED = '&#8833'
SUBSET_OF = '&#8834'
SUPERSET_OF = '&#8835'
NOT_A_SUBSET_OF = '&#8836'
NOT_A_SUPERSET_OF = '&#8837'
SUBSET_OF_OR_EQUAL_TO = '&#8838'
SUPERSET_OF_OR_EQUAL_TO = '&#8839'
NEITHER_A_SUBSET_OF_NOR_EQUAL_TO = '&#8840'
NEITHER_A_SUPERSET_OF_NOR_EQUAL_TO = '&#8841'
SUBSET_OF_WITH_NOT_EQUAL_TO = '&#8842'
SUPERSET_OF_WITH_NOT_EQUAL_TO = '&#8843'
MULTISET = '&#8844'
MULTISET_MULTIPLICATION = '&#8845'
MULTISET_UNION = '&#8846'
SQUARE_IMAGE_OF = '&#8847'
SQUARE_ORIGINAL_OF = '&#8848'
SQUARE_IMAGE_OF_OR_EQUAL_TO = '&#8849'
SQUARE_ORIGINAL_OF_OR_EQUAL_TO = '&#8850'
SQUARE_CAP = '&#8851'
SQUARE_CUP = '&#8852'
CIRCLED_PLUS = '&#8853'
CIRCLED_MINUS = '&#8854'
CIRCLED_TIMES = '&#8855'
CIRCLED_DIVISION_SLASH = '&#8856'
CIRCLED_DOT_OPERATOR = '&#8857'
CIRCLED_RING_OPERATOR = '&#8858'
CIRCLED_ASTERISK_OPERATOR = '&#8859'
CIRCLED_EQUALS = '&#8860'
CIRCLED_DASH = '&#8861'
SQUARED_PLUS = '&#8862'
SQUARED_MINUS = '&#8863'
SQUARED_TIMES = '&#8864'
SQUARED_DOT_OPERATOR = '&#8865'
RIGHT_TACK = '&#8866'
LEFT_TACK = '&#8867'
DOWN_TACK = '&#8868'
UP_TACK = '&#8869'
ASSERTION = '&#8870'
MODELS = '&#8871'
TRUE = '&#8872'
FORCES = '&#8873'
TRIPLE_VERTICAL_BAR_RIGHT_TURNSTILE = '&#8874'
DOUBLE_VERTICAL_BAR_DOUBLE_RIGHT_TURNSTILE = '&#8875'
DOES_NOT_PROVE = '&#8876'
NOT_TRUE = '&#8877'
DOES_NOT_FORCE = '&#8878'
NEGATED_DOUBLE_VERTICAL_BAR_DOUBLE_RIGHT_TURNSTILE = '&#8879'
PRECEDES_UNDER_RELATION = '&#8880'
SUCCEEDS_UNDER_RELATION = '&#8881'
NORMAL_SUBGROUP_OF = '&#8882'
CONTAINS_AS_NORMAL_SUBGROUP = '&#8883'
NORMAL_SUBGROUP_OF_OR_EQUAL_TO = '&#8884'
CONTAINS_AS_NORMAL_SUBGROUP_OR_EQUAL_TO = '&#8885'
ORIGINAL_OF = '&#8886'
IMAGE_OF = '&#8887'
MULTIMAP = '&#8888'
HERMITIAN_CONJUGATE_MATRIX = '&#8889'
INTERCALATE = '&#8890'
XOR = '&#8891'
NAND = '&#8892'
NOR = '&#8893'
RIGHT_ANGLE_WITH_ARC = '&#8894'
RIGHT_TRIANGLE = '&#8895'
N_ARY_LOGICAL_AND = '&#8896'
N_ARY_LOGICAL_OR = '&#8897'
N_ARY_INTERSECTION = '&#8898'
N_ARY_UNION = '&#8899'
DIAMOND_OPERATOR = '&#8900'
DOT_OPERATOR = '&#8901'
STAR_OPERATOR = '&#8902'
DIVISION_TIMES = '&#8903'
BOWTIE = '&#8904'
LEFT_NORMAL_FACTOR_SEMIDIRECT_PRODUCT = '&#8905'
RIGHT_NORMAL_FACTOR_SEMIDIRECT_PRODUCT = '&#8906'
LEFT_SEMIDIRECT_PRODUCT = '&#8907'
RIGHT_SEMIDIRECT_PRODUCT = '&#8908'
REVERSED_TILDE_EQUALS = '&#8909'
CURLY_LOGICAL_OR = '&#8910'
CURLY_LOGICAL_AND = '&#8911'
DOUBLE_SUBSET = '&#8912'
DOUBLE_SUPERSET = '&#8913'
DOUBLE_INTERSECTION = '&#8914'
DOUBLE_UNION = '&#8915'
PITCHFORK = '&#8916'
EQUAL_AND_PARALLEL_TO = '&#8917'
LESS_THAN_WITH_DOT = '&#8918'
GREATER_THAN_WITH_DOT = '&#8919'
VERY_MUCH_LESS_THAN = '&#8920'
VERY_MUCH_GREATER_THAN = '&#8921'
LESS_THAN_EQUAL_TO_OR_GREATER_THAN = '&#8922'
GREATER_THAN_EQUAL_TO_OR_LESS_THAN = '&#8923'
EQUAL_TO_OR_LESS_THAN = '&#8924'
EQUAL_TO_OR_GREATER_THAN = '&#8925'
EQUAL_TO_OR_PRECEDES = '&#8926'
EQUAL_TO_OR_SUCCEEDS = '&#8927'
DOES_NOT_PRECEDE_OR_EQUAL = '&#8928'
DOES_NOT_SUCCEED_OR_EQUAL = '&#8929'
NOT_SQUARE_IMAGE_OF_OR_EQUAL_TO = '&#8930'
NOT_SQUARE_ORIGINAL_OF_OR_EQUAL_TO = '&#8931'
SQUARE_IMAGE_OF_OR_NOT_EQUAL_TO = '&#8932'
SQUARE_ORIGINAL_OF_OR_NOT_EQUAL_TO = '&#8933'
LESS_THAN_BUT_NOT_EQUIVALENT_TO = '&#8934'
GREATER_THAN_BUT_NOT_EQUIVALENT_TO = '&#8935'
PRECEDES_BUT_NOT_EQUIVALENT_TO = '&#8936'
SUCCEEDS_BUT_NOT_EQUIVALENT_TO = '&#8937'
NOT_NORMAL_SUBGROUP_OF = '&#8938'
DOES_NOT_CONTAIN_AS_NORMAL_SUBGROUP = '&#8939'
NOT_NORMAL_SUBGROUP_OF_OR_EQUAL_TO = '&#8940'
DOES_NOT_CONTAIN_AS_NORMAL_SUBGROUP_OR_EQUAL = '&#8941'
VERTICAL_ELLIPSIS = '&#8942'
MIDLINE_HORIZONTAL_ELLIPSIS = '&#8943'
UP_RIGHT_DIAGONAL_ELLIPSIS = '&#8944'
DOWN_RIGHT_DIAGONAL_ELLIPSIS = '&#8945'
ELEMENT_OF_WITH_LONG_HORIZONTAL_STROKE = '&#8946'
ELEMENT_OF_WITH_VERTICAL_BAR_AT_END_OF_HORIZONTAL_STROKE = '&#8947'
SMALL_ELEMENT_OF_WITH_VERTICAL_BAR_AT_END_OF_HORIZONTAL_STROKE = '&#8948'
ELEMENT_OF_WITH_DOT_ABOVE = '&#8949'
ELEMENT_OF_WITH_OVERBAR = '&#8950'
SMALL_ELEMENT_OF_WITH_OVERBAR = '&#8951'
ELEMENT_OF_WITH_UNDERBAR = '&#8952'
ELEMENT_OF_WITH_TWO_HORIZONTAL_STROKES = '&#8953'
CONTAINS_WITH_LONG_HORIZONTAL_STROKE = '&#8954'
CONTAINS_WITH_VERTICAL_BAR_AT_END_OF_HORIZONTAL_STROKE = '&#8955'
SMALL_CONTAINS_WITH_VERTICAL_BAR_AT_END_OF_HORIZONTAL_STROKE = '&#8956'
CONTAINS_WITH_OVERBAR = '&#8957'
SMALL_CONTAINS_WITH_OVERBAR = '&#8958'
Z_NOTATION_BAG_MEMBERSHIP = '&#8959'